#include "stdafx.h"
#include "PopWatch.h"
#include "Settings.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//Pull in the OpenSSL libraries
#pragma comment(lib, "ssleay32.lib")
#pragma comment(lib, "libeay32.lib")

const TCHAR g_pszXorData[] = _T("PopWatch DoDataExchange_&352364");

CCriticalSection* CPopWatchApp::m_pOpenSSLCritSections = NULL;

CPopWatchApp* GetApp()
{
  return (CPopWatchApp*) AfxGetApp();
}

CMainFrame* GetMainFrame()
{
  return (CMainFrame*) AfxGetMainWnd();
}

CPopWatchApp theApp;



class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

protected:
  HICON m_hIcon;

	//{{AFX_VIRTUAL(CAboutDlg)
	virtual void DoDataExchange(CDataExchange* pDX);
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	afx_msg HCURSOR OnQueryDragIcon();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT

  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CAboutDlg::OnInitDialog() 
{
  //Let the parent class do its thing
	CDialog::OnInitDialog();
	
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	return TRUE;
}

HCURSOR CAboutDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}




BEGIN_MESSAGE_MAP(CPopWatchApp, CWinApp)
	//{{AFX_MSG_MAP(CPopWatchApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_APP_SETTINGS, OnAppSettings)
	ON_COMMAND(ID_HELP, OnHelp)
	ON_COMMAND(ID_APP_CHECKNOW, OnAppChecknow)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CPopWatchApp::CPopWatchApp()
{
  m_hSpawnedProcess = NULL;
}

void __cdecl CPopWatchApp::OpenSSLLockingCallback(int mode, int type, const char* /*file*/, int /*line*/)
{
  ASSERT(m_pOpenSSLCritSections);
	if (mode & CRYPTO_LOCK)
		m_pOpenSSLCritSections[type].Lock();
	else
		m_pOpenSSLCritSections[type].Unlock();
}

BOOL CPopWatchApp::InitInstance()
{
  //Usual MFC initialisation
  Enable3dControls();

  //Initialise the Winsock stack
  if (!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

  //Standard OpenSSL initialization
  SSL_load_error_strings();
  SSL_library_init();

  //Setup SSL to work correctly in a multithreaded environment
  ASSERT(m_pOpenSSLCritSections == NULL);
  m_pOpenSSLCritSections = new CCriticalSection[CRYPTO_num_locks()];
  if (m_pOpenSSLCritSections == NULL)
  {
    //Report the error
    TRACE(_T("Failed to create SSL critical sections\n"));
    return FALSE;
  }
  CRYPTO_set_locking_callback(OpenSSLLockingCallback);

  //Change the INI file to use to point to the directory where the EXE is located
  TCHAR pszFileName[_MAX_PATH];
  GetModuleFileName(NULL, pszFileName, _MAX_PATH);
  TCHAR drive[_MAX_DRIVE];   
  TCHAR dir[_MAX_DIR];
  TCHAR fname[_MAX_FNAME];   
  _tsplitpath(pszFileName, drive, dir, fname, NULL);
  _tmakepath(pszFileName, drive, dir, fname, _T("INI"));
  free((void*)m_pszProfileName);
  m_pszProfileName = _tcsdup(pszFileName);

  OSVERSIONINFO osvi;
  osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  m_bWin2k = (GetVersionEx(&osvi) && osvi.dwPlatformId == VER_PLATFORM_WIN32_NT && osvi.dwMajorVersion >= 5);

  //Load up the settings from the ini file
  LoadSettings();

  //Create the main window initially hidden
  CMainFrame* pMainFrame = new CMainFrame;
  m_pMainWnd = pMainFrame;
  if (!pMainFrame->Create(NULL, _T("PopWatch Hidden Wnd")))
    return FALSE;

  m_pMainWnd->ShowWindow(SW_HIDE);
  m_pMainWnd->UpdateWindow();

	return TRUE;
}

void CPopWatchApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

TCHAR* CPopWatchApp::EncryptString(const CString& sString)
{
  int nLength = sString.GetLength();
  TCHAR* pData = new TCHAR[nLength];

  //Xor the data with the mask string
  for (int i=0; i<nLength; i++)
    pData[i] = (TCHAR) (sString.GetAt(i) ^ g_pszXorData[i % _tcslen(g_pszXorData)]);

  return pData;
}

BOOL CPopWatchApp::IsProcessRunning(HANDLE hProcess)
{
	return (WaitForSingleObject(hProcess, 0) == WAIT_TIMEOUT);
}

HANDLE CPopWatchApp::StartProcessGettingProcessHandle(const TCHAR* pszCommandLine)
{
	SHELLEXECUTEINFO sei;
	ZeroMemory(&sei, sizeof(SHELLEXECUTEINFO));
	sei.cbSize = sizeof (SHELLEXECUTEINFO);
	sei.lpVerb = NULL;
	sei.lpFile = pszCommandLine;
	sei.nShow = SW_SHOW;
	sei.hInstApp = NULL;
	sei.lpDirectory = NULL;
	sei.fMask = SEE_MASK_DOENVSUBST|SEE_MASK_NOCLOSEPROCESS;
	sei.lpParameters = NULL;
	if (ShellExecuteEx (&sei) )
		return sei.hProcess;
	else	
    return NULL;
}

BOOL CPopWatchApp::StartProcessWithoutGettingProcessHandle(const TCHAR* pszCommandLine)
{
	SHELLEXECUTEINFO sei;
	ZeroMemory(&sei, sizeof(SHELLEXECUTEINFO));
	sei.cbSize = sizeof (SHELLEXECUTEINFO);
	sei.lpVerb = NULL;
	sei.lpFile = pszCommandLine;
	sei.nShow = SW_SHOW;
	sei.hInstApp = NULL;
	sei.lpDirectory = NULL;
	sei.fMask = SEE_MASK_DOENVSUBST;
	sei.lpParameters = NULL;
	return ShellExecuteEx(&sei);
}

CString CPopWatchApp::DecryptString(const BYTE* pData, int nSize)
{
  TCHAR* pszString = new TCHAR[nSize+1];

  //Xor the data with the mask string
  for (int i=0; i<nSize; i++)
    pszString[i] = (TCHAR) (pData[i] ^ g_pszXorData[i % _tcslen(g_pszXorData)]);
  pszString[nSize] = _T('\0'); //Do not forget to NULL terminate

  CString sString(pszString);
  delete [] pszString;
  return sString;
}

void CPopWatchApp::LoadSettings()
{
  //Load the values up from the ini file
  m_nInterval = GetProfileInt(_T("General"), _T("Interval"), 5);
  m_bCheckUsingRas = (BOOL) GetProfileInt(_T("General"), _T("UseRas"), FALSE);
  m_bBalloon = (BOOL) GetProfileInt(_T("General"), _T("Balloon"), m_bWin2k);
  m_bOneInstance = (BOOL) GetProfileInt(_T("General"), _T("OneInstance"), FALSE);
  m_sProgramToRun = GetProfileString(_T("General"), _T("ProgramToRun"), _T(""));
  m_sWaveFileToPlay = GetProfileString(_T("General"), _T("WaveFileToPlay"), _T(""));

  int nAccounts = GetProfileInt(_T("General"), _T("Accounts"), 0);
  m_sPasswords.SetSize(0, nAccounts);
  m_sPop3Servers.SetSize(0, nAccounts);
  m_sUsernames.SetSize(0, nAccounts);
  m_sAccountNames.SetSize(0, nAccounts);
  m_sWaveFiles.SetSize(0, nAccounts);
  m_nPorts.SetSize(0, nAccounts);
  m_bSSLs.SetSize(0, nAccounts);
  m_bDisables.SetSize(0, nAccounts);
  for (int i=0; i<nAccounts; i++)
  {
    CString sSection;
    sSection.Format(_T("Account%d"), i);

    LPBYTE pEncryptedPassword = NULL;
    UINT nPasswordSize = 0;
	  GetProfileBinary(sSection, _T("Password"),	&pEncryptedPassword, &nPasswordSize);
    CString sPassword = DecryptString(pEncryptedPassword, nPasswordSize);
    delete [] pEncryptedPassword;

    m_sPasswords.Add(sPassword);
    m_sPop3Servers.Add(GetProfileString(sSection, _T("Server"), _T("")));
    m_sUsernames.Add(GetProfileString(sSection, _T("Username"), _T("")));
    m_sAccountNames.Add(GetProfileString(sSection, _T("Name"), _T("")));
    m_sWaveFiles.Add(GetProfileString(sSection, _T("WaveFileToPlay"), _T("")));
    m_nPorts.Add(GetProfileInt(sSection, _T("Port"), 110));
    m_bSSLs.Add((BOOL) GetProfileInt(sSection, _T("SSL"), FALSE));
    m_bDisables.Add((BOOL) GetProfileInt(sSection, _T("Disable"), FALSE));
  }
}

void CPopWatchApp::SaveSettings()
{
  //Save the values in the ini file
  WriteProfileInt(_T("General"), _T("Interval"), m_nInterval);
  WriteProfileInt(_T("General"), _T("UseRas"), m_bCheckUsingRas);
  WriteProfileInt(_T("General"), _T("Balloon"), m_bBalloon);
  WriteProfileInt(_T("General"), _T("OneInstance"), m_bOneInstance);
  WriteProfileString(_T("General"), _T("ProgramToRun"), m_sProgramToRun);
  WriteProfileString(_T("General"), _T("WaveFileToPlay"), m_sWaveFileToPlay);

  int nAccounts = m_sAccountNames.GetSize();
  WriteProfileInt(_T("General"), _T("Accounts"), nAccounts);
  for (int i=0; i<nAccounts; i++)
  {
    CString sSection;
    sSection.Format(_T("Account%d"), i);

    CString sPassword = m_sPasswords.GetAt(i);
    TCHAR* pEncryptedPassword = EncryptString(sPassword);
    WriteProfileBinary(sSection, _T("Password"), (LPBYTE) pEncryptedPassword, sPassword.GetLength());
    delete [] pEncryptedPassword;

    WriteProfileString(sSection, _T("Server"), m_sPop3Servers.GetAt(i));
    WriteProfileString(sSection, _T("Username"), m_sUsernames.GetAt(i));
    WriteProfileString(sSection, _T("Name"), m_sAccountNames.GetAt(i));
    WriteProfileString(sSection, _T("WaveFileToPlay"), m_sWaveFiles.GetAt(i));
    WriteProfileInt(sSection, _T("Port"), m_nPorts.GetAt(i));
    WriteProfileInt(sSection, _T("SSL"), m_bSSLs.GetAt(i));
    WriteProfileInt(sSection, _T("Disable"), m_bDisables.GetAt(i));
  }
}

void CPopWatchApp::OnAppSettings() 
{
	CSettingsPropertySheet propSheet;
  propSheet.m_psh.dwFlags |= PSH_NOAPPLYNOW; 

  //Load the values up from the member variables
  propSheet.m_Page1.m_nInterval = m_nInterval;
  propSheet.m_Page1.m_sPasswords.Copy(m_sPasswords);
  propSheet.m_Page1.m_sPop3Servers.Copy(m_sPop3Servers);
  propSheet.m_Page1.m_sUsernames.Copy(m_sUsernames);
  propSheet.m_Page1.m_sAccountNames.Copy(m_sAccountNames);
  propSheet.m_Page1.m_sWaveFiles.Copy(m_sWaveFiles);
  propSheet.m_Page1.m_nPorts.Copy(m_nPorts);
  propSheet.m_Page1.m_bSSLs.Copy(m_bSSLs);
  propSheet.m_Page1.m_bDisables.Copy(m_bDisables);

  propSheet.m_Page1.m_bCheckUsingRas = m_bCheckUsingRas;
  propSheet.m_Page1.m_bBalloon = m_bBalloon;
  propSheet.m_Page1.m_sWaveFile = m_sWaveFileToPlay;
  propSheet.m_Page1.m_sProgram = m_sProgramToRun;
  propSheet.m_Page1.m_bOneInstance = m_bOneInstance;

  //Bring up the tabbed dialog to allow the values to be edited
	if (propSheet.DoModal() == IDOK)
  {
    //Update the member variables
    m_nInterval = propSheet.m_Page1.m_nInterval;
    m_sPasswords.Copy(propSheet.m_Page1.m_sPasswords);
    m_sPop3Servers.Copy(propSheet.m_Page1.m_sPop3Servers);
    m_sUsernames.Copy(propSheet.m_Page1.m_sUsernames);
    m_sAccountNames.Copy(propSheet.m_Page1.m_sAccountNames);
    m_sWaveFiles.Copy(propSheet.m_Page1.m_sWaveFiles);
    m_nPorts.Copy(propSheet.m_Page1.m_nPorts);
    m_bSSLs.Copy(propSheet.m_Page1.m_bSSLs);
    m_bDisables.Copy(propSheet.m_Page1.m_bDisables);

    m_bCheckUsingRas = propSheet.m_Page1.m_bCheckUsingRas;
    m_bBalloon = propSheet.m_Page1.m_bBalloon;
    m_sWaveFileToPlay = propSheet.m_Page1.m_sWaveFile;
    m_sProgramToRun = propSheet.m_Page1.m_sProgram;
    m_bOneInstance = propSheet.m_Page1.m_bOneInstance;

    //Save the values in the ini file
    SaveSettings();

    //Inform te mainframe of the changes
    GetMainFrame()->OnSettingsUpdate();
  }
}

void CPopWatchApp::OnHelp() 
{
  TCHAR sFilename[_MAX_PATH];
  GetModuleFileName(NULL, sFilename, _MAX_PATH);
  TCHAR sDrive[_MAX_DRIVE];
  TCHAR sDir[_MAX_DIR];
  TCHAR sFname[_MAX_FNAME];
  _tsplitpath(sFilename, sDrive, sDir, sFname, NULL);
  _tmakepath(sFilename, sDrive, sDir, sFname, _T("HTM"));

	HINSTANCE hRun = ShellExecute(AfxGetMainWnd()->m_hWnd, _T("open"), sFilename, NULL, NULL, SW_SHOW);
  if ((int) hRun <= 32)
    AfxMessageBox(IDS_FAIL_RUN_HELP, MB_OK | MB_ICONEXCLAMATION);
}

void CPopWatchApp::OnAppChecknow() 
{
	GetMainFrame()->CheckNow();	
}

int CPopWatchApp::ExitInstance() 
{
  //Free up the process handle we have spawned if valid
  if (m_hSpawnedProcess)
  {
    CloseHandle(m_hSpawnedProcess);
    m_hSpawnedProcess = NULL;  
  }

  //Clean up OpenSSL library
  ERR_free_strings();
  ERR_remove_state(0);
  EVP_cleanup();
  CRYPTO_cleanup_all_ex_data();

  //Clean up the SSL critical sections
  if (m_pOpenSSLCritSections)
  {
    delete [] m_pOpenSSLCritSections;
    m_pOpenSSLCritSections = NULL;
  }

  //Let the base class do its thing	
	return CWinApp::ExitInstance();
}


