/*
Module : RASMONITOR.H
Purpose: Defines the interface for a number of wrapper classes for 
         monitoring Ras connections 

Copyright (c) 1998 - 2003 by PJ Naughter.  (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/

#ifndef __RASMONITOR_H__
#define __RASMONITOR_H__
                      
#ifndef __AFXTEMPL_H__
#pragma message("To avoid this message please put afxtempl.h in your PCH")                                                                                
#include <afxtempl.h>
#endif

#ifndef _WINSVC_
#pragma message("To avoid this message please put winsvc.h in your PCH")                                                                                
#include <winsvc.h>
#endif

#ifndef TAPI_H
#pragma message("To avoid this message please put tapi.h in your PCH")                                                                                
#include <Tapi.h>
#endif


///////////////////// Includes //////////////////////////////////////

#include "cpdh.h"     //my own wrapper class for performance data on NT
#include <pdhmsg.h>


//////// forward class reference ////////////////////////////////////
class CRasMonitor;


//A class represeing a Dial-Up Networking connection
class CRasConnection
{
public:
//Constructors / Destructors
  CRasConnection(CRasMonitor* pMonitor);
	CRasConnection(CRasMonitor* pmonitor, const RASCONN& Connection);
  ~CRasConnection();

//Methods
  void Serialize(CArchive& ar);
  static void SerializeTime(CArchive& ar, SYSTEMTIME& st);
  void GetInitialConnectionDetails();
  void GetCurrentConnectionDetails();

//Serialized Data
  CString    m_sName;                    //Name of the connection
	CString    m_sDeviceName;              //Name of the Device this connection is using
	CString    m_sDeviceType;              //Name of the Device Type this connection is using
  SYSTEMTIME m_ConnectionTime;           //The local time when the connection was started
  SYSTEMTIME m_DialTime;                 //The local time when the connection was dialled
  BOOL       m_bConnected;               //True if currently connected, FALSE during the dialing
                                         //stage of a connection
  DWORD      m_ConnectDuration;          //how long the connection was made (in Seconds)
  DWORD      m_DialDuration;             //how long the connection took to be made (in Seconds)
  SYSTEMTIME m_HangupTime;               //The local time when the connection was closed
  DWORD      m_dwBytesReceived;          //The total number of bytes received
  DWORD      m_dwBytesSent;              //The number of bytes sent
  DWORD      m_dwConnectionSpeed;        //The connection speed
  //Useful information from TAPI's LINECALLINFo structure
  CString    m_sCallerIDNumber;          //The caller's number
  CString    m_sCallerIDName;            //The caller's name
  CString    m_sCalledIDNumber;          //The called number  
  CString    m_sCalledIDName;            //The called name
  CString    m_sConnectedIDNumber;       //The connected number (may be different than destination if call is diverted)  
  CString    m_sConnectedIDName;         //The connected name
  CString    m_sDisplayableAddress;      //Displayable string for logging purposes 
  CString    m_sCalledParty;             //User friendly description of the called party
  CString    m_sComment;                 //Comment about the call

protected:
//Temporary Data
  DWORD        m_dwInitialBytesReceived; //The total number of bytes received when connection is made
  DWORD        m_dwInitialBytesSent;     //The number of bytes sent received when connection is made
  HRASCONN     m_hRasConn;               //Ras API handle for this conenction

//Other data
  CRasMonitor* m_pMonitor;               //Pointer to the main ras monitor class instance 

//Misc methods
  void Init(CRasMonitor* pMonitor);
  void GetDetailsViaRasGetConnectionStatistics();

#ifndef RAS_MONITOR_NO_PDH
  CPdhCounter* m_pNTcounterR;            //NT performance counter for bytes received
  CPdhCounter* m_pNTcounterT;            //NT performance counter for bytes transmitted
  CPdhQuery    m_NTquery;                //NT performance query 
#endif
};


//WPARAM which is used in notification message
const WPARAM RAS_DIAL_EVENT = 1;
const WPARAM RAS_CONNECT_EVENT = 2;
const WPARAM RAS_DISCONNECT_EVENT = 3;
const WPARAM RAS_CHECK_EVENT = 4;


//The Ras Monitor class which client applications should 
//derive from and override the virtual functions to perform
//whatever application specific behaviour they want
class CRasMonitor : public CWnd
{
public:
//Constructors / Destructors
  CRasMonitor();
  virtual ~CRasMonitor();

//Methods
  BOOL RunningNT() const { return m_bRunningNT; };                     //Are we running on the "NT" Kernel
  BOOL Start(CWnd* pNotifyWnd, UINT nNotifyMessage);                   //Start monitoring ras connections
  BOOL Stop();                                                         //Stop monitoring ras connections 
  BOOL IsConnected(HRASCONN hConnection);                              //Determines if a ras connection handle is currently connected
  void Serialize(CArchive& ar);                                        //Serializes the current state of the connections to and from disk 
  int NumberOfConnections() const { return m_Connections.GetSize(); }; //Returns the current number of connections

protected:
//Define the RAS_STATS structure (which is also defined in SDK header file "Ras.h")
//This means that the code does not require you to have to have the latest 
//Platform SDK installed
  typedef struct _RAS_STATS
  {
    DWORD dwSize;
    DWORD dwBytesXmited;
    DWORD dwBytesRcved;
    DWORD dwFramesXmited;
    DWORD dwFramesRcved;
    DWORD dwCrcErr;
    DWORD dwTimeoutErr;
    DWORD dwAlignmentErr;
    DWORD dwHardwareOverrunErr;
    DWORD dwFramingErr;
    DWORD dwBufferOverrunErr;
    DWORD dwCompressionRatioIn;
    DWORD dwCompressionRatioOut;
    DWORD dwBps;
    DWORD dwConnectDuration;
  } RAS_STATS, *PRAS_STATS;

  //Function pointer typedefs used internally by CRasMonitor class
  typedef DWORD (__stdcall *PFNRASENUMCONNECTIONS)(LPRASCONN, LPDWORD, LPDWORD);
  typedef DWORD (__stdcall *PFNRASGETCONNECTSTATUS)(HRASCONN, LPRASCONNSTATUS);
  typedef DWORD (__stdcall *PFNRASGERCONNECTIONSTATISTICS)(HRASCONN, CRasMonitor::RAS_STATS*);

//Data
  CArray<CRasConnection*, CRasConnection*&> m_Connections;
  UINT                                      m_nTimerID;
  CWnd*                                     m_pNotificationWnd;
  UINT                                      m_nNotifyMessage;
  BOOL                                      m_bStarted;
  BOOL                                      m_bArchivedDataLoaded;
  SYSTEMTIME                                m_archiveTime;
  SC_HANDLE                                 m_hRasMan;
  PFNRASENUMCONNECTIONS                     m_pfnRasEnumConnections;
  PFNRASGETCONNECTSTATUS                    m_pfnRasGetConnectStatus;
  PFNRASGERCONNECTIONSTATISTICS             m_pfnRasGetConnectionStatistics;
  HINSTANCE                                 m_hRasApi32;
  BOOL                                      m_bRunningNT;
  DWORD                                     m_dwNumTapiDevs;
  HLINEAPP                                  m_hLineApp;

//Methods
  BOOL  GetRasDLLEntryPoints();
	BOOL  ConnectionExists(const CString& sName);
  int   GetConnectionIndex(const CString& sName);
  DWORD RasEnumConnections(LPRASCONN lprasconn, LPDWORD lpcb, LPDWORD lpcConnections); 
  DWORD RasGetConnectStatus(HRASCONN hrasconn, LPRASCONNSTATUS lprasconnstatus);
#ifndef RAS_MONITOR_NO_TAPI
  static void CALLBACK lineCallbackFunc(DWORD dwDevice, DWORD dwMsg, DWORD dwCallbackInstance, DWORD dwParam1, DWORD dwParam2, DWORD dwParam3);
  BOOL GetRasTAPIDetails(const CString& sDeviceName, BOOL bGetConnectionSpeed, CRasConnection& connection);
  BOOL InitializeTapi();
  void TerminateTapi();
#endif

//Virtual methods
	virtual void DoCheck(BOOL bClosing);
  virtual void OnDial(const CRasConnection& connection);
  virtual void OnConnect(const CRasConnection& connection);
  virtual void OnDisconnect(const CRasConnection& connection);
  virtual void OnCheck(BOOL bClosing, RASCONN* pConnections, DWORD dwConnections);
  virtual UINT GetTimerID();
#ifndef RAS_MONITOR_NO_PDH
  virtual CString GetPDHTotalBytesTxCounterName(); //Allows the PDH counter names to be overridden
  virtual CString GetPDHTotalBytesRxCounterName();
#endif
#ifndef RAS_MONITOR_NO_DYNDATA
  virtual CString GetDynDataTotalBytesRecvdCounterName();
  virtual CString GetDynDataTotalBytesSentCounterName();
  virtual CString CRasMonitor::GetDynDataCounterName();
#endif

	//{{AFX_MSG(CRasMonitor)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnTimer(UINT nIDEvent);
	//}}AFX_MSG
  DECLARE_MESSAGE_MAP()

  DECLARE_DYNAMIC(CRasMonitor)

  friend class CRasConnection;
};

#endif //__RASMONITOR_H__