#include "stdafx.h"
#include "resource.h"
#include "settings.h"
#include "popwatch.h"
#include "DlgAccount.h"


#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif



IMPLEMENT_DYNCREATE(CGeneralSettingsPage, CPropertyPage)

CGeneralSettingsPage::CGeneralSettingsPage() : CPropertyPage(CGeneralSettingsPage::IDD)
{
	//{{AFX_DATA_INIT(CGeneralSettingsPage)
	m_nInterval = 0;
	m_bCheckUsingRas = FALSE;
	m_bBalloon = FALSE;
	m_sProgram = _T("");
	m_sWaveFile = _T("");
	m_bOneInstance = FALSE;
	//}}AFX_DATA_INIT

  //Set some reasonable defaults
  m_nInterval = 5;

  if (!GetApp()->m_bWin2k)
    m_bBalloon = FALSE;
}

CGeneralSettingsPage::~CGeneralSettingsPage()
{
}

void CGeneralSettingsPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CGeneralSettingsPage)
	DDX_Control(pDX, IDC_ACCOUNTS, m_ctrlAccounts);
	DDX_Control(pDX, IDC_SPIN_INTERVAL, m_ctrlSpinInterval);
	DDX_Text(pDX, IDC_INTERVAL, m_nInterval);
	DDV_MinMaxInt(pDX, m_nInterval, 1, 60);
	DDX_Check(pDX, IDC_RAS, m_bCheckUsingRas);
	DDX_Check(pDX, IDC_BALLOON, m_bBalloon);
	DDX_Text(pDX, IDC_PROGRAM, m_sProgram);
	DDX_Text(pDX, IDC_WAVEFILE, m_sWaveFile);
	DDX_Check(pDX, IDC_ONE_INSTANCE, m_bOneInstance);
	//}}AFX_DATA_MAP
  DDX_FilenameControl(pDX, IDC_PROGRAM, m_ctrlProgram, GF_FILEMUSTEXIST, _T("Please specify a program to run"), _T("Executable Files (*.exe)|*.exe||"));
  DDX_FilenameControl(pDX, IDC_WAVEFILE, m_ctrlWaveFile, GF_FILEMUSTEXIST, _T("Please specify a sound file to play"), _T("Wave Files (*.wav)|*.wav||"));

  if (!pDX->m_bSaveAndValidate)
  {
    GetDlgItem(IDC_BALLOON)->EnableWindow(GetApp()->m_bWin2k);

    //Add all the accounts to the list box
    for (int i=0; i<m_sAccountNames.GetSize(); i++)
      m_ctrlAccounts.AddString(m_sAccountNames.GetAt(i));
  }
}

BEGIN_MESSAGE_MAP(CGeneralSettingsPage, CPropertyPage)
	//{{AFX_MSG_MAP(CGeneralSettingsPage)
	ON_BN_CLICKED(IDC_ADD, OnAdd)
	ON_BN_CLICKED(IDC_DELETE, OnDelete)
	ON_BN_CLICKED(IDC_EDIT, OnEdit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CGeneralSettingsPage::OnInitDialog() 
{
  //Let the parent do its thing
	CPropertyPage::OnInitDialog();
	
  //Set the spin range
  m_ctrlSpinInterval.SetRange(1, 60);

  //Select the first account by default
  m_ctrlAccounts.SetCurSel(0);
  	
	return TRUE;
}

CString CGeneralSettingsPage::CreateNewAccountName()
{
  //What will be the return value
  CString sAccountName;

  BOOL bUnique = FALSE;
  int nIndex = 1;
  int nAccounts = m_sAccountNames.GetSize();
  while (!bUnique)
  {
    //Form a potential new account name
    sAccountName.Format(_T("Account %d"), nIndex);

    //Check for uniqueness
    if (nAccounts)
    {
      //Assume the best before we start to loop thro
      bUnique = TRUE;
      for (int i=0; i<nAccounts && bUnique; i++)
        bUnique = (m_sAccountNames.GetAt(i).CompareNoCase(sAccountName) != 0);
    }
    else
      bUnique = TRUE;

    //Prepare for the next time around
    if (!bUnique)
      ++nIndex;
  }

  return sAccountName;
}

void CGeneralSettingsPage::OnAdd() 
{
  CDlgAccount dlg;
  dlg.m_sAccountName = CreateNewAccountName();

  for (int i=0; i<m_sAccountNames.GetSize(); i++)
    dlg.m_sExistingAccountNames.Add(m_sAccountNames.GetAt(i));

  if (dlg.DoModal() == IDOK)
  {
    //Add to the in memory arrays
    m_sAccountNames.Add(dlg.m_sAccountName);
    m_sPop3Servers.Add(dlg.m_sPop3Server);
    m_sUsernames.Add(dlg.m_sUsername);
    m_sPasswords.Add(dlg.m_sPassword);
    m_sWaveFiles.Add(dlg.m_sWaveFile);
    m_nPorts.Add(dlg.m_nPort);
    m_bSSLs.Add(dlg.m_bSSL);
    m_bDisables.Add(dlg.m_bDisable);

    //Add to the UI
    m_ctrlAccounts.AddString(dlg.m_sAccountName);
  }
}

void CGeneralSettingsPage::OnDelete() 
{
	int nCurAccount = m_ctrlAccounts.GetCurSel();
  if (nCurAccount != LB_ERR)
  {
    //Remove from the in memory arrrays
    m_sAccountNames.RemoveAt(nCurAccount);
    m_sPop3Servers.RemoveAt(nCurAccount);
    m_sUsernames.RemoveAt(nCurAccount);
    m_sPasswords.RemoveAt(nCurAccount);
    m_nPorts.RemoveAt(nCurAccount);
    m_bSSLs.RemoveAt(nCurAccount);
    m_bDisables.RemoveAt(nCurAccount);

    //Remove from the UI
    m_ctrlAccounts.DeleteString(nCurAccount);
  }
}

void CGeneralSettingsPage::OnEdit() 
{
	int nCurAccount = m_ctrlAccounts.GetCurSel();
  if (nCurAccount != LB_ERR)
  {
    CDlgAccount dlg;
    dlg.m_sAccountName = m_sAccountNames.GetAt(nCurAccount);
    dlg.m_sPop3Server  = m_sPop3Servers.GetAt(nCurAccount);
    dlg.m_sUsername    = m_sUsernames.GetAt(nCurAccount);
    dlg.m_sPassword    = m_sPasswords.GetAt(nCurAccount);
    dlg.m_sWaveFile    = m_sWaveFiles.GetAt(nCurAccount);
    dlg.m_nPort        = m_nPorts.GetAt(nCurAccount);
    dlg.m_bSSL         = m_bSSLs.GetAt(nCurAccount);
    dlg.m_bDisable     = m_bDisables.GetAt(nCurAccount);

    for (int i=0; i<m_sAccountNames.GetSize(); i++)
    {
      if (i != nCurAccount)
        dlg.m_sExistingAccountNames.Add(m_sAccountNames.GetAt(i));
    }

    if (dlg.DoModal() == IDOK)
    {
      //Update the in memory arrays
      m_sAccountNames.SetAt(nCurAccount, dlg.m_sAccountName);
      m_sPop3Servers.SetAt(nCurAccount, dlg.m_sPop3Server);
      m_sUsernames.SetAt(nCurAccount, dlg.m_sUsername);
      m_sPasswords.SetAt(nCurAccount, dlg.m_sPassword);
      m_sWaveFiles.SetAt(nCurAccount, dlg.m_sWaveFile);
      m_nPorts.SetAt(nCurAccount, dlg.m_nPort);
      m_bSSLs.SetAt(nCurAccount, dlg.m_bSSL);
      m_bDisables.SetAt(nCurAccount, dlg.m_bDisable);

      //Update in the UI CListBox
      m_ctrlAccounts.DeleteString(nCurAccount);
      m_ctrlAccounts.InsertString(nCurAccount, dlg.m_sAccountName);
    }
  }
}




IMPLEMENT_DYNAMIC(CSettingsPropertySheet, CPropertySheet)

CSettingsPropertySheet::CSettingsPropertySheet(CWnd* pWndParent)
	 : CPropertySheet(IDS_SETTINGS_CAPTION, pWndParent)
{
	AddPage(&m_Page1);
}

CSettingsPropertySheet::~CSettingsPropertySheet()
{
}

BEGIN_MESSAGE_MAP(CSettingsPropertySheet, CPropertySheet)
	//{{AFX_MSG_MAP(CSettingsPropertySheet)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

int CSettingsPropertySheet::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CPropertySheet::OnCreate(lpCreateStruct) == -1)
		return -1;
	
  //Setup the icon for the window, so that it appears
  //correctly when you Alt-Tab
  SetIcon(AfxGetApp()->LoadIcon(IDR_MAINFRAME), TRUE);	
	
	return 0;
}
